@@ -2,11 +2,10 @@ import sys
 import time
 import warnings
 import weakref
-from abc import ABCMeta
 from concurrent.futures import Future
 from contextlib import suppress
 from threading import Event, Thread
-from typing import Dict, Optional, Set
+from typing import Dict, Optional, Set, Type
 
 if sys.version_info >= (3, 10):
     # WPS433: Found nested import
@@ -19,55 +18,24 @@ else:
 
 # Annotations only
 with suppress(ImportError):
+    from command_based_framework.commands import (  # pragma: no cover
+        CommandState as _CommandState,
+    )
     from command_based_framework.actions import Action, Condition  # pragma: no cover
-    from command_based_framework.commands import Command  # pragma: no cover
     from command_based_framework.subsystems import Subsystem  # pragma: no cover
 
+from command_based_framework._common import CallableCommandType, CommandType
 from command_based_framework.exceptions import SchedulerExistsError
 
-ConditionCommandType: TypeAlias = Dict["Condition", Set["Command"]]
+CommandState: Type["_CommandState"]
+ConditionCommandType: TypeAlias = Dict["Condition", Set[CallableCommandType]]
 ActionStack: TypeAlias = Dict["Action", ConditionCommandType]
 
-
-class SchedulerMeta(ABCMeta, type):
-    """Meta attributes for :py:class:`~command_based_framework.scheduler.Scheduler`.
-
-    All methods and attributes here exist in :py:class:`~command_based_framework.scheduler.Scheduler`.
-    """  # noqa: E501
-
-    _instance: Optional[weakref.ReferenceType["Scheduler"]]
-
-    @property
-    def instance(cls) -> Optional["Scheduler"]:
-        """The class-level instance attribute.
-
-        Ensures that no more than one scheduler instance exists else
-        undefined behavior regarding subsystems may occur.
-
-        :return: The :py:class:`~command_based_framework.scheduler.Scheduler`
-            instance if set, otherwise `None`.
-
-        :raise SchedulerExistsError:
-            An attempt was made to create multiple :py:class:`~command_based_framework.scheduler.Scheduler`
-            instances.
-        """  # noqa: E501
-        if cls._instance is None:
-            return None
-
-        return cls._instance()
-
-    @instance.setter
-    def instance(cls, instance: "Scheduler") -> None:
-        # Verify no previous instance exists
-        if Scheduler.instance:
-            raise SchedulerExistsError(
-                "a scheduler already exists, a new one cannot be created",
-            )
-        cls._instance = weakref.ref(instance)
+_INSTANCE: Optional["weakref.ref[Scheduler]"] = None
 
 
-class Scheduler(object, metaclass=SchedulerMeta):
-    """Schedules :py:class:`~command_based_framework.commands.Command` that control :py:class:`~command_based_framework.subsystems.Subsystem` when activated by :py:class:`~command_based_framework.actions.Action`.
+class Scheduler(object):
+    """Event loop of the framework.
 
     The scheduler handles events and resource management for the
     framework. A scheduler **must** exist **before** creating any
@@ -77,55 +45,55 @@ class Scheduler(object, metaclass=SchedulerMeta):
 
     The scheduler has the following life-cycle:
 
-    **1**: All actions with bound `when` methods have their :py:meth:`~command_based_framework.actions.Action.poll`
-    methods called. For efficiency, unbound actions are ignored.
+    - All actions with bound `when` methods have their
+        :meth:`~command_based_framework.actions.Action.poll` methods
+        called. For efficiency, unbound actions are ignored.
+    - If a `when` condition is met, the related command(s) are
+        put onto the incoming stack. Any two commands which share
+        requirements (subsystems) will result in the currently scheduled
+        command being interrupted. Two newly scheduled commands with
+        shared requirements will result in only one of the commands
+        being scheduled and a warning being thrown about the conflict.
+    - Any subsystem not bound to a scheduled or incoming command
+        will have their default command scheduled, if there is one.
+    - Commands scheduled for interruption are interrupted. Commands
+        scheduled to exit because their
+        :meth:`~command_based_framework.commands.Command.is_finished`
+        returned `True`, are ended. Incoming commands are initialized.
+        Note that commands initialized in one frame will not be normally
+        executed until the next frame.
+    - Currently scheduled commands have their
+        :meth:`~command_based_framework.commands.Command.is_finished`
+        methods checked. If `True` is returned, the command(s) are
+        scheduled for exit and their
+        :meth:`~command_based_framework.commands.Command.execute` method
+        is ignored. Otherwise, their
+        :meth:`~command_based_framework.commands.Command.execute` method
+        is called. Commands are allowed to raise errors from their
+        :meth:`~command_based_framework.commands.Command.execute`
+        methods and their
+        :meth:`~command_based_framework._common.ContextManagerMixin.handle_exception`
+        method will be called with the output from :meth:`sys.exec_info`.
+        Return `True` to indicate the error is handled and normal
+        execution can continue. Any other return will result in the
+        command being immediately interrupted and de-stacked. A warning
+        about the error will be thrown.
+    - The main stack updates with interrupted/finished commands taken
+        off and initialized commands put on.
+    - This cycle repeats.
+    - Upon shutdown, all current commands are interrupted and
+        de-stacked. The scheduler then exits its event loop.
 
-    **2**: If a `when` condition is met, the related command(s) are
-    put onto the incoming stack. Any two commands which share
-    requirements (subsystems) will result in the currently scheduled
-    command being interrupted. Two newly scheduled commands with shared
-    requirements will result in only one of the commands being scheduled
-    and a warning being thrown about the conflict.
-
-    **3**: Any subsystem not bound to a scheduled or incoming command
-    will have their default command scheduled, if there is one.
-
-    **4**: Commands scheduled for interruption are interrupted. Commands
-    scheduled to exit because their :py:meth:`~command_based_framework.commands.Command.is_finished`
-    returned `True`, are ended. Incoming commands are initialized. Note
-    that commands initialized in one frame will not be normally executed
-    until the next frame.
-
-    **5**: Currently scheduled commands have their :py:meth:`~command_based_framework.commands.Command.is_finished`
-    methods checked. If `True` is returned, the command(s) are scheduled
-    for exit and their :py:meth:`~command_based_framework.commands.Command.execute`
-    method is ignored. Otherwise, their :py:meth:`~command_based_framework.commands.Command.execute`
-    method is called. Commands are allowed to raise errors from their
-    :py:meth:`~command_based_framework.commands.Command.execute` methods
-    and their :py:meth:`~command_based_framework.commands.Command.handle_exception`
-    method will be called with the output from :py:meth:`sys.exec_info`.
-    Return `True` to indicate the error is handled and normal execution
-    can continue. Any other return will result in the command being
-    immediately interrupted and de-stacked. A warning about the error
-    will be thrown.
-
-    **6**: The main stack updates with interrupted/finished commands
-    taken off and initialized commands put on.
-
-    **7**: This cycle repeats.
-
-    **8**: Upon shutdown, all current commands are interrupted and
-    de-stacked. The scheduler then exits its event loop.
     """  # noqa: E501
 
-    _instance: Optional[weakref.ReferenceType["Scheduler"]] = None
+    _instance: Optional["weakref.ReferenceType[Scheduler]"] = None
 
     # Clock speed is how fast the scheduler runs per second
     _clock_speed: float
 
     # All stack has references to all commands in any stack, regardless
     # of status
-    _all_stack: Set["Command"]
+    _all_stack: Set[CallableCommandType]
 
     # Action stack has references to the mappings between commands and
     # actions
@@ -133,15 +101,19 @@ class Scheduler(object, metaclass=SchedulerMeta):
 
     # Incoming stack has references to all commands that were just
     # scheduled
-    _incoming_stack: Set["Command"]
+    _incoming_stack: Set[CallableCommandType]
 
     # Scheduled stack has references to all commands that are normally
     # executing
-    _scheduled_stack: Set["Command"]
+    _scheduled_stack: Set[CommandType]
 
     # Ended stack has references to all commands that need to be ended
     # normally
-    _ended_stack: Set["Command"]
+    _ended_stack: Set[CommandType]
+
+    # Cancel stack has references to all commands that need to be
+    # canceled
+    _cancel_stack: Set[CommandType]
 
     # Subsystem stack has references to all subsystems that need to
     # have their periodic methods called
@@ -154,25 +126,39 @@ class Scheduler(object, metaclass=SchedulerMeta):
     _exec_sentinel: Event
 
     def __init__(self) -> None:
-        """Creates a new :py:class:`~command_based_framework.scheduler.Scheduler` instance."""
-        Scheduler.instance = self
+        """Creates a new :class:`Scheduler` instance."""
+        # Check for existing instances
+        global _INSTANCE  # noqa: WPS420
+        if _INSTANCE and _INSTANCE():
+            raise SchedulerExistsError(
+                "a scheduler already exists, a new one cannot be created",
+            )
+
+        # Set the global instance
+        _INSTANCE = weakref.ref(self)  # noqa: WPS122, WPS442
+
+        # Continue with creation
+        super().__init__()
         self.clock_speed = 1 / 60
         self._reset_all_stacks()
         self._exec_sentinel = Event()
 
+        # Prevent circular import
+        from command_based_framework.commands import CommandState as _CS  # noqa: N814
+
+        global CommandState  # noqa: WPS420
+        CommandState = _CS  # noqa: WPS442
+
     @property
     def clock_speed(self) -> float:
         """How many times the scheduler will attempt to run per second.
 
         Because the scheduler is synchronous, long-running commands may
         degrade the ability for the scheduler to stick to this rate.
-        This value must always remain above 0 otherwise the CPU may be
-        deadlocked.
+        This value must always remain above 0 otherwise a
+        :exc:`ValueError` is raised to prevent CPU deadlock.
 
         Defaults to 60 ticks per second.
-
-        :raise ValueError: An attempt to set the clock speed at or below
-            0 was made.
         """
         return self._clock_speed
 
@@ -184,10 +170,18 @@ class Scheduler(object, metaclass=SchedulerMeta):
 
         self._clock_speed = clock_speed
 
+    @classmethod
+    def get_instance(cls) -> Optional["Scheduler"]:  # noqa: WPS615
+        """Get the global scheduler instance."""
+        if _INSTANCE is not None:
+            return _INSTANCE()
+
+        return None
+
     def bind_command(
         self,
         action: "Action",
-        command: "Command",
+        command: CallableCommandType,
         condition: "Condition",
     ) -> None:
         """Bind `command` to `action` to be scheduled on `condition`."""
@@ -205,21 +199,31 @@ class Scheduler(object, metaclass=SchedulerMeta):
         self._actions_stack[action] = current_condition_stack
         self._all_stack.add(command)
 
-    def cancel(self, *commands: "Command") -> None:  # noqa: WPS213
+    def cancel(self, *commands: CommandType) -> None:  # noqa: C901, WPS213, WPS231
         """Immediately cancel and interrupt any number of commands.
 
         If `commands` is not provided, interrupt all scheduled and
         incoming commands. The `interrupt` parameter of the
-        :py:meth:`~command_based_framework.commands.Command.end` method
+        :meth:`~command_based_framework.commands.Command.end` method
         for each command will be `True`.
 
-        :param commands: Variable length of commands to cancel. If not
-            provided, interrupt all active commands.
-        :type commands: tuple
+        Args:
+            commands: Variable length of commands to cancel. If not
+                provided, interrupt all scheduled and initialized
+                commands.
         """
         cancel_all = not commands
         all_commands = set(commands) or self._all_stack
         for command in all_commands.copy():
+            # Ignore commands which are callables
+            if callable(command):
+                continue
+
+            # Ignore commands that are idle
+            if command.state == CommandState.idle:
+                continue
+
+            command.state = CommandState.idle
             try:
                 command.end(interrupted=True)
             except Exception:
@@ -240,8 +244,6 @@ class Scheduler(object, metaclass=SchedulerMeta):
                 requirement.current_command = None
 
             # Remove the command from all stacks
-            with suppress(KeyError):
-                self._all_stack.remove(command)
             with suppress(KeyError):
                 self._incoming_stack.remove(command)
             with suppress(KeyError):
@@ -255,16 +257,16 @@ class Scheduler(object, metaclass=SchedulerMeta):
     def execute(self, fork: bool = False) -> Optional[Future]:
         """Perpetually run the event loop.
 
-        :param fork: Fork a separate thread to run the event loop in. If
-            `True`, a :py:class:`~concurrent.futures.Future` is
-            returned. If `False` and an attempt is made to shut the
-            event loop down via :py:meth:`~command_based_framework.scheduler.Scheduler.shutdown`
-            in the same thread, a deadlock will occur.
-        :type fork: bool
+        Args:
+            fork: Fork a separate thread to run the event loop in. If
+                `True`, a :class:`~concurrent.futures.Future` is
+                returned. If `False` and an attempt is made to shut the
+                event loop down via :meth:`~.Scheduler.shutdown`
+                in the same thread, a deadlock will occur.
 
-        :return: A :py:class:`~concurrent.futures.Future` if `fork` is
-            `True`, otherwise `None`.
-        :rtype: :py:class:`~concurrent.futures.Future`, None
+        Returns:
+            A :class:`~concurrent.futures.Future` if `fork` is
+                `True`, otherwise `None` upon exit.
         """  # noqa: DAR202
         # Reset the sentinel
         self._exec_sentinel.clear()
@@ -279,29 +281,29 @@ class Scheduler(object, metaclass=SchedulerMeta):
         return self._execute()  # type: ignore
 
     def prestart_setup(self) -> None:
-        """Run prestart checks and setup when :py:meth:`~command_based_framework.scheduler.Scheduler.execute` is called."""  # noqa: E501
+        """Run prestart checks and setup when :meth:`~.Scheduler.execute` is called."""  # noqa: E501
 
     def postend_teardown(self) -> None:
-        """Run post-end code and teardown when :py:meth:`~command_based_framework.scheduler.Scheduler.execute` exits."""  # noqa: E501
+        """Run post-end code and teardown when :meth:`~.Scheduler.execute` exits."""  # noqa: E501
 
     def register_subsystem(self, subsystem: "Subsystem") -> None:
-        """Register a :py:class:`~command_based_framework.subsystems.Subsystem` with the scheduler.
+        """Register a :class:`~command_based_framework.subsystems.Subsystem` with the scheduler.
 
         This should be called automatically by the subsystem upon
         creation, so calling this directly should not be necessary.
         Registered subsystems allow for default commands to be scheduled
         if the subsystem is not active in another command.
 
-        :param subsystem: The subsystem to register.
-        :type subsystem: :py:class:`~command_based_framework.subsystems.Subsystem`
+        Args:
+            subsystem: The subsystem to register.
         """  # noqa: E501
         self._subsystem_stack.add(subsystem)
 
     def run_once(self) -> None:
         """Run one complete loop of the scheduler's event loop.
 
-        Note this does not call :py:meth:`~Scheduler.prestart_setup` or
-        :py:meth:`~Scheduler.postend_teardown`.
+        Note this does not call :meth:`~Scheduler.prestart_setup` or
+        :meth:`~Scheduler.postend_teardown`.
         """
         self._poll_actions()
         self._schedule_default_commands()
@@ -351,14 +353,17 @@ class Scheduler(object, metaclass=SchedulerMeta):
             self.postend_teardown()
 
             # Set the future as finished
-            if fut:
+            if fut and not fut.done():
                 fut.set_result(None)
 
     def _end_commands(self) -> None:
+        # End commands normally
         for cmd in self._ended_stack:
+            cmd.state = CommandState.idle
             with cmd:
                 cmd.end(interrupted=False)
-            self._scheduled_stack.remove(cmd)
+            with suppress(KeyError):
+                self._scheduled_stack.remove(cmd)
 
             # Reset all requirements' current commands to none
             for requirement in cmd.requirements:
@@ -367,10 +372,15 @@ class Scheduler(object, metaclass=SchedulerMeta):
             # Reset the interrupt flag
             cmd.needs_interrupt  # noqa: WPS428
 
+        # Cancel commands
+        if self._cancel_stack:
+            self.cancel(*self._cancel_stack)
+
     def _execute_commands(self) -> None:
         for command in self._scheduled_stack.copy():
             # Check if the command is finished before executing it
             # Safer to do this check first
+            command.state = CommandState.executing
             if command.is_finished():
                 with command:
                     command.end(interrupted=False)
@@ -399,7 +409,14 @@ class Scheduler(object, metaclass=SchedulerMeta):
                 subsystem.periodic()
 
     def _init_commands(self) -> None:  # noqa: C901, WPS231
-        for command in self._incoming_stack.copy():
+        for command_callable in self._incoming_stack.copy():
+            # If the "command" is a callable, run it and get the output
+            command: CommandType = (
+                command_callable() if callable(command_callable) else command_callable
+            )
+            self._incoming_stack.remove(command_callable)
+            self._incoming_stack.add(command)
+
             # If the command is already scheduled, don't init
             if command in self._scheduled_stack:
                 self._incoming_stack.remove(command)
@@ -412,7 +429,11 @@ class Scheduler(object, metaclass=SchedulerMeta):
                 if cmd == command:
                     continue
 
-                if cmd.requirements.intersection(command.requirements):
+                # Skip functions
+                if callable(command):
+                    continue  # type: ignore
+
+                if cmd.requirements.intersection(command.requirements):  # type: ignore
                     self._incoming_stack.remove(command)
                     skip = True
             if skip:
@@ -433,6 +454,7 @@ class Scheduler(object, metaclass=SchedulerMeta):
 
             # Initialize this command
             with command:
+                command.state = CommandState.initialized
                 command.initialize()
 
             if command.needs_interrupt:
@@ -456,6 +478,13 @@ class Scheduler(object, metaclass=SchedulerMeta):
 
             # Handle each state type
             if action_state == (False, True):
+                # Cancel when activated
+                commands = conditions_commands.setdefault(
+                    Condition.cancel_when_activated,
+                    set(),
+                )
+                self._cancel_stack.update(commands)  # type: ignore
+
                 # When activated
                 commands = conditions_commands.setdefault(
                     Condition.when_activated,
@@ -470,7 +499,7 @@ class Scheduler(object, metaclass=SchedulerMeta):
                 )
                 for command in commands:
                     if command in self._scheduled_stack:
-                        self._ended_stack.add(command)
+                        self._ended_stack.add(command)  # type: ignore
                     else:
                         self._incoming_stack.add(command)
             elif action_state == (True, True):
@@ -489,7 +518,7 @@ class Scheduler(object, metaclass=SchedulerMeta):
                 # Need to be able to deactivate when held, which occurs
                 # at the same time as when deactivated
                 commands = conditions_commands.setdefault(Condition.when_held, set())
-                self._ended_stack.update(commands)
+                self._ended_stack.update(commands)  # type: ignore
 
     def _reset_all_stacks(self) -> None:
         self._all_stack = set()
@@ -497,12 +526,17 @@ class Scheduler(object, metaclass=SchedulerMeta):
         self._incoming_stack = set()
         self._scheduled_stack = set()
         self._ended_stack = set()
+        self._cancel_stack = set()
         self._subsystem_stack = set()
 
     def _schedule_default_commands(self) -> None:  # noqa: WPS231
         for subsystem in self._subsystem_stack:
             if subsystem.default_command and not subsystem.current_command:
                 for command in self._incoming_stack.union(self._scheduled_stack):
+                    # Ignore commands which are callables
+                    if callable(command):
+                        continue
+
                     if subsystem in command.requirements:
                         break
                 else:
@@ -511,11 +545,13 @@ class Scheduler(object, metaclass=SchedulerMeta):
     def _update_stack(self) -> None:
         # Remove interrupted and ended commands
         self._scheduled_stack.difference_update(self._ended_stack)
+        self._scheduled_stack.difference_update(self._cancel_stack)
 
         # Move incoming commands to scheduled stack
-        self._scheduled_stack.update(self._incoming_stack)
+        self._scheduled_stack.update(self._incoming_stack)  # type: ignore
         self._all_stack.update(self._scheduled_stack)
 
         # Reset incoming, ended, and interrupted stacks
         self._incoming_stack.clear()
         self._ended_stack.clear()
+        self._cancel_stack.clear()
