@@ -2,10 +2,11 @@ import math
 from threading import Thread
 import time
 
+import mock
 import pytest
 
 from command_based_framework.actions import Action, Condition
-from command_based_framework.commands import Command, ParallelCommandGroup, SequentialCommandGroup
+from command_based_framework.commands import Command, CommandState, ParallelCommandGroup, SequentialCommandGroup
 from command_based_framework.scheduler import Scheduler
 from command_based_framework.subsystems import Subsystem
 
@@ -15,11 +16,11 @@ def test_tracking_scheduler_instances() -> None:
     from command_based_framework.exceptions import SchedulerExistsError
 
     # Verify no scheduler has been set
-    assert Scheduler.instance == None
+    assert Scheduler.get_instance() == None
 
     # Create a scheduler and verify it is tracked
     s = Scheduler()
-    assert Scheduler.instance == s
+    assert Scheduler.get_instance() == s
 
     # Ensure no new scheduler can be created
     with pytest.raises(SchedulerExistsError):
@@ -36,7 +37,7 @@ def test_tracking_scheduler_instances() -> None:
 
 def test_setting_clock_speed() -> None:
     """Verify the clock speed is set properly."""
-    scheduler = Scheduler.instance or Scheduler()
+    scheduler = Scheduler.get_instance() or Scheduler()
 
     assert math.isclose(scheduler.clock_speed, 1 / 60)
 
@@ -56,7 +57,7 @@ def test_setting_clock_speed() -> None:
 
 def test_rebinding_same_command() -> None:
     """Verify actions are bound to commands correctly."""
-    scheduler = Scheduler.instance or Scheduler()
+    scheduler = Scheduler.get_instance() or Scheduler()
     scheduler._reset_all_stacks()
 
     # Verify the stack is empty
@@ -96,7 +97,7 @@ def test_rebinding_same_command() -> None:
 
 def test_binding_multiple_commands_same_action() -> None:
     """Verify multiple commands bind to an action."""
-    scheduler = Scheduler.instance or Scheduler()
+    scheduler = Scheduler.get_instance() or Scheduler()
     scheduler._reset_all_stacks()
 
     # Verify the stack is empty
@@ -132,7 +133,7 @@ def test_binding_multiple_commands_same_action() -> None:
 
 def test_rebinding_multiple_commands_same_action() -> None:
     """Verify rebinding multiple commands on the same action."""
-    scheduler = Scheduler.instance or Scheduler()
+    scheduler = Scheduler.get_instance() or Scheduler()
     scheduler._reset_all_stacks()
 
     # Verify the stack is empty
@@ -178,7 +179,7 @@ def test_rebinding_multiple_commands_same_action() -> None:
 
 def test_binding_multiple_actions() -> None:
     """Verify binding a command to multiple actions."""
-    scheduler = Scheduler.instance or Scheduler()
+    scheduler = Scheduler.get_instance() or Scheduler()
     scheduler._reset_all_stacks()
 
     # Verify the stack is empty
@@ -214,16 +215,12 @@ def test_binding_multiple_actions() -> None:
 
 def test_cancel_command() -> None:
     """Verify commands are canceled."""
-    scheduler = Scheduler.instance or Scheduler()
+    scheduler = Scheduler.get_instance() or Scheduler()
     scheduler._reset_all_stacks()
 
     # Verify the stack is empty
     assert not scheduler._actions_stack
 
-    class MyAction(Action):
-
-        def poll(self) -> bool:
-            return True
 
     class MyCommand(Command):
 
@@ -239,8 +236,8 @@ def test_cancel_command() -> None:
             self.canceled_counter += 1
             assert interrupted
 
-    action = MyAction()
     command = MyCommand()
+    command.state = CommandState.executing
 
     # Inject the command into the scheduler's stack
     scheduler._all_stack.add(command)
@@ -256,13 +253,14 @@ def test_cancel_command() -> None:
 
 def test_command_raises_runtime_warning_in_cancel() -> None:
     """Verify commands raise RuntimeWarnings if they fail to cancel."""
-    scheduler = Scheduler.instance or Scheduler()
+    scheduler = Scheduler.get_instance() or Scheduler()
     scheduler._reset_all_stacks()
 
     # Verify the stack is empty
     assert not scheduler._actions_stack
 
     class MyCommand(Command):
+        _state = CommandState.executing
 
         def is_finished(self) -> bool:
             return True
@@ -274,6 +272,7 @@ def test_command_raises_runtime_warning_in_cancel() -> None:
             raise ValueError("test error")
 
     command = MyCommand()
+    command.state = CommandState.executing
 
     # Artificially inject the command into the stack
     scheduler._all_stack.add(command)
@@ -285,6 +284,7 @@ def test_command_raises_runtime_warning_in_cancel() -> None:
 
     # Re-inject the command
     scheduler._all_stack.add(command)
+    command.state = CommandState.executing
 
     # Verify the command fails and raises a warning if explicitly
     # specified
@@ -294,7 +294,7 @@ def test_command_raises_runtime_warning_in_cancel() -> None:
 
 def test_scheduler_event_loop() -> None:
     """Verify the event loop schedules everything correctly."""
-    scheduler = Scheduler.instance or Scheduler()
+    scheduler = Scheduler.get_instance() or Scheduler()
     scheduler._reset_all_stacks()
 
     # Verify the stack is empty
@@ -538,7 +538,7 @@ def test_scheduler_event_loop() -> None:
 
 def test_toggle_commands() -> None:
     """Verify commands toggle."""
-    scheduler = Scheduler.instance or Scheduler()
+    scheduler = Scheduler.get_instance() or Scheduler()
     scheduler._reset_all_stacks()
 
     # Verify the stack is empty
@@ -630,7 +630,7 @@ def test_toggle_commands() -> None:
 
 def test_command_error_cancels() -> None:
     """Verify commands cancel if methods raise unhandable exceptions."""
-    scheduler = Scheduler.instance or Scheduler()
+    scheduler = Scheduler.get_instance() or Scheduler()
     scheduler._reset_all_stacks()
 
     # Verify the stack is empty
@@ -687,7 +687,7 @@ def test_command_error_cancels() -> None:
 
 def test_conflicting_incoming_commands() -> None:
     """Verify incoming commands with conflicting requirements."""
-    scheduler = Scheduler.instance or Scheduler()
+    scheduler = Scheduler.get_instance() or Scheduler()
     scheduler._reset_all_stacks()
 
     # Verify the stack is empty
@@ -751,7 +751,7 @@ def test_conflicting_incoming_commands() -> None:
 
 def test_scheduled_incoming_conflicting_commands() -> None:
     """Verify scheduled commands are interrupted by incoming commands."""
-    scheduler = Scheduler.instance or Scheduler()
+    scheduler = Scheduler.get_instance() or Scheduler()
     scheduler._reset_all_stacks()
 
     # Verify the stack is empty
@@ -798,8 +798,10 @@ def test_scheduled_incoming_conflicting_commands() -> None:
     subsystem = MySubsystem()
     subsystem2 = MySubsystem()
     command1 = MyCommand("Command1", subsystem)
+    command1.state = CommandState.executing
     command2 = MyCommand("Command2", subsystem)
     command3 = MyCommand("Command3", subsystem2)
+    command3.state = CommandState.executing
 
     # Inject one command into the scheduled stack and the other into the
     # incoming stack
@@ -815,9 +817,13 @@ def test_scheduled_incoming_conflicting_commands() -> None:
     assert command3.did_exec
 
 
-def test_subsystems_dont_default_incoming_commands() -> None:
+@mock.patch.object(Command, "initialize")
+@mock.patch.object(Command, "execute")
+@mock.patch.object(Command, "is_finished")
+@mock.patch.object(Command, "end")
+def test_subsystems_dont_default_incoming_commands(*_) -> None:
     """Verify subsystems detect incoming commands and don't default."""
-    scheduler = Scheduler.instance or Scheduler()
+    scheduler = Scheduler.get_instance() or Scheduler()
     scheduler._reset_all_stacks()
 
     # Verify the stack is empty
@@ -865,9 +871,13 @@ def test_subsystems_dont_default_incoming_commands() -> None:
     assert subsystem.current_command == None
 
 
-def test_forked_exec() -> None:
+@mock.patch.object(Command, "initialize")
+@mock.patch.object(Command, "execute")
+@mock.patch.object(Command, "is_finished")
+@mock.patch.object(Command, "end")
+def test_forked_exec(*_) -> None:
     """Verify execute runs normally when forked."""
-    scheduler = Scheduler.instance or Scheduler()
+    scheduler = Scheduler.get_instance() or Scheduler()
     scheduler._reset_all_stacks()
 
     # Verify the stack is empty
@@ -922,9 +932,13 @@ def test_forked_exec() -> None:
     assert command1.did_exec
 
 
-def test_exec() -> None:
+@mock.patch.object(Command, "initialize")
+@mock.patch.object(Command, "execute")
+@mock.patch.object(Command, "is_finished")
+@mock.patch.object(Command, "end")
+def test_exec(*_) -> None:
     """Verify execute runs when executed in the main thread."""
-    scheduler = Scheduler.instance or Scheduler()
+    scheduler = Scheduler.get_instance() or Scheduler()
     scheduler._reset_all_stacks()
 
     # Verify the stack is empty
@@ -981,9 +995,13 @@ def test_exec() -> None:
 
 
 @pytest.mark.filterwarnings("ignore::pytest.PytestUnhandledThreadExceptionWarning")
-def test_exec_cancels_on_error() -> None:
+@mock.patch.object(Command, "initialize")
+@mock.patch.object(Command, "execute")
+@mock.patch.object(Command, "is_finished")
+@mock.patch.object(Command, "end")
+def test_exec_cancels_on_error(*_) -> None:
     """Verify the event loop cancels all commands when errors occur."""
-    scheduler = Scheduler.instance or Scheduler()
+    scheduler = Scheduler.get_instance() or Scheduler()
     scheduler._reset_all_stacks()
 
     # Verify the stack is empty
@@ -1068,7 +1086,7 @@ def test_exec_cancels_on_error() -> None:
 
 def test_sequential_command_group() -> None:
     """Verify sequential command group operates as expected."""
-    scheduler = Scheduler.instance or Scheduler()
+    scheduler = Scheduler.get_instance() or Scheduler()
     scheduler._reset_all_stacks()
 
     # Verify the stack is empty
@@ -1157,6 +1175,9 @@ def test_sequential_command_group() -> None:
     assert command1.did_exec == 1
     assert command1.did_finish == 2
     assert command1.did_end == 1
+
+    scheduler.run_once()
+
     assert command2.did_init == 1
 
     scheduler.run_once()
@@ -1201,7 +1222,7 @@ def test_sequential_command_group() -> None:
 
 def test_parallel_command_group() -> None:
     """Verify parallel command groups execute as expected."""
-    scheduler = Scheduler.instance or Scheduler()
+    scheduler = Scheduler.get_instance() or Scheduler()
     scheduler._reset_all_stacks()
 
     # Verify the stack is empty
@@ -1225,7 +1246,6 @@ def test_parallel_command_group() -> None:
         raise_error = False
 
         def handle_exception(self, *exc) -> bool:
-            print(exc)
             return True
 
         def initialize(self) -> None:
@@ -1341,13 +1361,14 @@ def test_parallel_command_group() -> None:
 
     scheduler.run_once()
 
-    # All threads to execute for a little bit
+    # Allow threads to execute for a little bit
     time.sleep(1)
 
     # Raise an error in command 5
     command5.raise_error = True
 
-    time.sleep(20)
+    # Allow threads to catch error being raised
+    time.sleep(0.1)
 
     # Ensure the group catches the error and handles it properly
     scheduler.run_once()
@@ -1363,3 +1384,97 @@ def test_parallel_command_group() -> None:
     assert command6.did_finish > 0
     assert command6.did_end == 0
     assert command6.did_interrupt == 1
+
+
+def test_callable_command_type() -> None:
+    """Verify callables that return commands are executed correctly."""
+    scheduler = Scheduler.get_instance() or Scheduler()
+    scheduler._reset_all_stacks()
+
+    # Verify the stack is empty
+    assert not scheduler._actions_stack
+
+    class MyAction(Action):
+
+        state = False
+
+        def poll(self) -> bool:
+            return self.state
+
+    class MyCommand(Command):
+
+        did_init = 0
+        did_exec = 0
+        did_end = 0
+        did_interrupt = 0
+        did_finish = 0
+        do_finish = False
+
+        def initialize(self) -> None:
+            self.did_init += 1
+
+        def execute(self) -> None:
+            self.did_exec += 1
+
+        def end(self, interrupted: bool) -> None:
+            if interrupted:
+                self.did_interrupt += 1
+                return
+            self.did_end += 1
+
+        def is_finished(self) -> bool:
+            self.did_finish += 1
+            return self.do_finish
+
+        def reset(self) -> None:
+            self.did_init = 0
+            self.did_exec = 0
+            self.did_end = 0
+            self.did_interrupt = 0
+            self.do_finish = False
+
+    class MySubsystem(Subsystem):
+
+        periodic_counter = 0
+
+        def periodic(self) -> None:
+            self.periodic_counter += 1
+
+        def reset(self) -> None:
+            self.periodic_counter = 0
+
+    action = MyAction()
+    subsystem = MySubsystem()
+    command1 = MyCommand("Command1", subsystem)
+    command2 = MyCommand("Command2", subsystem)
+
+    action.when_activated(lambda: (
+        SequentialCommandGroup(
+            None,
+            command1,
+            command2,
+        )
+    ))
+
+    # Activate the action
+    action.state = True
+
+    scheduler.run_once()
+
+    assert isinstance(subsystem.current_command, SequentialCommandGroup)
+    assert command1.did_init == 1
+
+    scheduler.run_once()
+    assert command1.did_init == 1
+    assert command1.did_exec == 1
+    assert command1.did_finish == 1
+
+    command1.do_finish = True
+
+    scheduler.run_once()
+
+    assert command1.did_end == 1
+
+    scheduler.run_once()
+
+    assert command2.did_init == 1
